@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2006 Double Precision, Inc.
+** Copyright 1998 - 2011 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -532,7 +532,7 @@ static void print_xtext(FILE *fp, const char *xtext)
 
 static void print_receipient(FILE *fp, struct ctlfile *ctf, unsigned n)
 {
-	putc('<', fp);
+	fputs(" <", fp);
 	if (ctf->oreceipients[n] &&
 #if	HAVE_STRNCASECMP
 		strncasecmp(ctf->oreceipients[n], "rfc822;", 7)
@@ -553,16 +553,16 @@ static void format_del_info(const char *p, const char type, void *q)
 	switch (type)	{
 	case COMCTLFILE_DELINFO_PEER:
 		if (*p)
-			fprintf((FILE *)q, "    %s:\n", p);
+			fprintf((FILE *)q, "     %s:\n", p);
 		break;
 	case COMCTLFILE_DELINFO_CONNECTIONERROR:
-		fprintf((FILE *)q, "*** %s\n", p);
+		fprintf((FILE *)q, " *** %s\n", p);
 		break;
 	case COMCTLFILE_DELINFO_SENT:
-		fprintf((FILE *)q, ">>> %s\n", p);
+		fprintf((FILE *)q, " >>> %s\n", p);
 		break;
 	case COMCTLFILE_DELINFO_REPLY:
-		fprintf((FILE *)q, "<<< %-1.500s\n", p);
+		fprintf((FILE *)q, " <<< %-1.500s\n", p);
 		break;
 	}
 }
