@@ -41,11 +41,11 @@
 #include	<courierauth.h>
 #include	<courierauthsasl.h>
 
-static const char rcsid[]="$Id: courieresmtpd.c,v 1.57 2010/02/02 22:30:00 mrsam Exp $";
 
 static char helobuf[256];
 static char authuserbuf[256];
 static char tlsbuf[128+NUMBUFSIZE];
+static int extended;
 
 static unsigned long sizelimit;
 static int submit_started=0;
@@ -146,8 +146,7 @@ const char *p;
 	} while (*banner);
 }
 
-static void ehlo(const char *heloname, int hastls, int tls,
-		 int extended)
+static void ehlo(const char *heloname, int hastls, int tls)
 {
 static const char e[]=
 COURIER_EXTENSIONS
@@ -289,12 +288,13 @@ int	rc;
 
 static void startsubmit(int tls)
 {
-char	*argv[13];
-const char *ident;
-char	*identbuf=0;
-int	n;
-const	char *host;
-char	*buf;
+	char	*argv[14];
+	const char *ident;
+	char	*identbuf=0;
+	int	n, exid_ndx=0;
+	const	char *host;
+	char	*buf;
+	char rfc3848_buf[32];
 
 	if (submit_started)	return;
 	if (helobuf[0] == '\0')	return;
@@ -315,8 +315,29 @@ char	*buf;
 			strcat(strcpy(authbuf, "-auth="), p+1);
 			argv[n++]=authbuf;
 		}
+		exid_ndx=1;
 	}
 
+	strcpy(rfc3848_buf, "-rfc3848=");
+
+	if (extended)
+	{
+		static char *exid[] =
+			{"ESMTP",
+			 "ESMTPA",
+			 "ESMTPS",
+			 "ESMTPSA"};
+
+		if (tls) exid_ndx += 2;
+		strcat(rfc3848_buf, exid[exid_ndx]);
+
+	}
+	else
+	{
+		strcat(rfc3848_buf, "SMTP");
+	}
+	argv[n++]=rfc3848_buf;
+
 	argv[n++]="esmtp";
 
 	host=tcpremotehost;
@@ -965,6 +986,18 @@ static int auth_callback_func(struct authinfo *a, void *va)
 	return 0;
 }
 
+static char *rtrim(char *s)
+{
+	char *end = s + strlen(s);
+	while (s < end)
+		if (isspace(*(unsigned char*)--end))
+			*end = 0;
+		else
+			break;
+	return s;
+}
+
+
 /*****************************************************************************
 
 courieresmtpd can be started in two situations:
@@ -1117,12 +1150,14 @@ int	authenticated=0;
 			*p=toupper(*p);
 		}
 
+		rtrim(line);
 		if (strcmp(line, "QUIT") == 0)	break;
 		if ((strncmp(line, "EHLO ", 5) == 0 ||
 			strncmp(line, "HELO ", 5) == 0) &&
 			line[5])
 		{
-			ehlo(line+5, starttls, tls, line[0] == 'E');
+			extended=line[0] == 'E';
+			ehlo(line+5, starttls, tls);
 			iovflush();
 			cancelsubmit();
 			startsubmit(tls);
@@ -1161,6 +1196,7 @@ int	authenticated=0;
 
 			addiovec("220 Ok\r\n", 8);
 			iovflush();
+			iovreset();
 
 			if (couriertls_start(argvec, &cinfo))
 			{
@@ -1220,6 +1256,9 @@ int	authenticated=0;
 		char	*buf=strcpy(courier_malloc(strlen(line)+1), line);
 		char	*authtype;
 		char	*authdata;
+		char    fakecmd[5];
+
+		        strcpy(fakecmd, "AUTH");
 
 			starttls=0;
 			(void)strtok(buf, " \t\r\n");
@@ -1235,6 +1274,7 @@ int	authenticated=0;
 
 			if (authmethod == 0 || *authmethod == 0)
 			{
+				input_line=fakecmd;
 				addiovec_error("535 Authentication rejected");
 				free(buf);
 				continue;
@@ -1277,6 +1317,7 @@ int	authenticated=0;
 
 			if (authuserbuf[0] == 0)
 			{
+				input_line=fakecmd;
 				addiovec_error("535 Authentication failed.");
 				iovflush();
 			}
