@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2008 Double Precision, Inc.
+** Copyright 1998 - 2010 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -285,12 +285,20 @@ static const char *want_security(struct ctlfile *ctf)
 	return (0);
 }
 
-static int get_sourceaddr(int af, RFC1035_NETADDR *addrbuf,
-	const struct sockaddr **addrptr, int *addrptrlen)
+static int get_sourceaddr(int af,
+			  const RFC1035_ADDR *dest_addr,
+			  RFC1035_NETADDR *addrbuf,
+			  const struct sockaddr **addrptr, int *addrptrlen)
 {
 	int rc;
 	RFC1035_ADDR in;
-	const char *buf=getenv("SOURCE_ADDRESS");
+	const char *buf=getenv(
+#if RFC1035_IPV6
+
+			       !IN6_IS_ADDR_V4MAPPED(dest_addr)
+			       ? "SOURCE_ADDRESS_IPV6":
+#endif
+			       "SOURCE_ADDRESS");
 
 	if (buf && strcmp(buf, "0")) {
 		rc = rfc1035_aton(buf, &in);
@@ -528,14 +536,14 @@ static void sendesmtp(struct moduledel *del, struct ctlfile *ctf)
 			is_secure_connection=0;
 			if ((sockfd=rfc1035_mksocket(SOCK_STREAM, 0, &af))
 					>= 0 &&
-				rfc1035_mkaddress(af, &addrbuf, &addr, port,
-					&addrptr, &addrptrlen) == 0 &&
-				get_sourceaddr(af, &saddrbuf, &saddrptr,
-					&saddrptrlen) == 0 &&
-				rfc1035_bindsource(sockfd, saddrptr,
-					saddrptrlen) == 0 &&
-				s_connect(sockfd, addrptr, addrptrlen,
-					  connect_timeout) == 0)
+			    rfc1035_mkaddress(af, &addrbuf, &addr, port,
+					      &addrptr, &addrptrlen) == 0 &&
+			    get_sourceaddr(af, &addr, &saddrbuf, &saddrptr,
+					   &saddrptrlen) == 0 &&
+			    rfc1035_bindsource(sockfd, saddrptr,
+					       saddrptrlen) == 0 &&
+			    s_connect(sockfd, addrptr, addrptrlen,
+				      connect_timeout) == 0)
 			{
 				/*
 				** If we're connected, make sure EHLO or HELO
@@ -1397,6 +1405,8 @@ static int hello2(struct moduledel *del, struct ctlfile *ctf, int using_tls)
 			hasstarttls=0;
 	}
 
+	if (getenv("COURIER_ESMTP_DEBUG_NO8BITMIME"))
+		has8bitmime=0;
 	return (0);
 }
 
