@@ -7,7 +7,6 @@
 #define	submit_h
 
 #include	"afx.h"
-#include	"afxtempl.h"
 #include	"dbobj.h"
 #include	"courier.h"
 #if	HAVE_SYS_STAT_H
@@ -19,7 +18,9 @@
 #include	<fstream>
 #include	<time.h>
 
-using namespace std;
+#include	<list>
+#include	<string>
+#include	<set>
 
 /////////////////////////////////////////////////////////////////////////////
 //
@@ -70,8 +71,8 @@ public:
 	unsigned num_receipient;
 
 	struct rw_transport *driver;
-	CString	host;
-	CString	address;
+	std::string host;
+	std::string address;
 	unsigned rcptnum;
 } ;
 
@@ -80,20 +81,21 @@ private:
 
 static	SubmitFile *current_submit_file;
 
-	CString	submit_filename_buf;
-	char	*submit_filename_buf_ptr;
 	unsigned num_control_files_created;
 
-	char *name1stctlfile();
-	char *namefile(const char *, unsigned n);
+	std::string name1stctlfile();
+	std::string namefile(const char *, unsigned n);
 
+ public:
+	std::set<std::string> all_files;
+ private:
 	unsigned	rcptcount;
 	afxopipestream	ctlfile;
 	afxiopipestream	datfile;
 	ino_t		ctlinodenum;
 	ino_t		ctlpid;
 	time_t		ctltimestamp;
-	CString		basemsgid;
+	std::string	basemsgid;
 
 	struct		rfc2045 *rwrfcptr;
 	const char 	*frommta;
@@ -101,13 +103,13 @@ static	SubmitFile *current_submit_file;
 	unsigned long	sizelimit;
 	int		diskfull;
 	unsigned	diskspacecheck;
-	CString	sender, envid, dsnformat;
-	CString receipient;
-	CMap<CString, CString, AFXBOOL, AFXBOOL> addrlist_map;
+	std::string	sender, envid, dsnformat;
+	std::string	receipient;
+	std::set<std::string> addrlist_map;
 	DbObj					addrlist_gdbm;
 
-	CList<RcptFilterInfo, const RcptFilterInfo &> rcptfilterlist;
-	fstream	rcptfilterlist_file;
+	std::list<RcptFilterInfo> rcptfilterlist;
+	std::fstream	rcptfilterlist_file;
 
 	const char *sending_module;
 
@@ -117,7 +119,7 @@ public:
 	SubmitFile();
 	~SubmitFile();
 
-	char* QueueID();
+	std::string QueueID();
 	void SendingModule(const char *p) {sending_module=p;}
 	void Sender(const char *, const char *, const char *, char);
 	int ChkRecipient(const char *);
@@ -131,18 +133,20 @@ public:
 	void MessageStart();
 	void Message(const char *);
 	int MessageEnd(unsigned, int, int);
-static CString get_msgid_for_filtering(unsigned, void *);
+
+	static std::string get_msgid_for_filtering(unsigned, void *);
 
 static	void interrupt();
 static	void trapsignals();
 
+
 private:
 	int datafilter(const char *, unsigned, const char *);
 
 	void do_datafilter( unsigned &, int &, int,
-		struct rw_transport *, CString, CString, unsigned,
+			    struct rw_transport *, std::string, std::string, unsigned,
 
-		unsigned, unsigned, const char *, unsigned nrcpts);
+			    unsigned, unsigned, const char *, unsigned nrcpts);
 } ;
 
 #endif
