@@ -1,8 +1,7 @@
 /*
-** Copyright 1998 - 2009 Double Precision, Inc.
+** Copyright 1998 - 2010 Double Precision, Inc.
 ** See COPYING for distribution information.
 **
-** $Id: pop3dserver.c,v 1.43 2009/03/23 11:00:44 mrsam Exp $
 */
 
 #if	HAVE_CONFIG_H
@@ -524,8 +523,8 @@ static void cleanup()
 {
 	unsigned i;
 
-	long deleted_bytes=0;
-	long deleted_messages=0;
+	int64_t deleted_bytes=0;
+	int64_t deleted_messages=0;
 
 	for (i=0; i<msglist_cnt; i++)
 		if (msglist_a[i]->isdeleted)
@@ -561,17 +560,11 @@ static void cleanup()
 	return;
 }
 
-static void printed(int cnt)
-{
-	if (cnt > 0)
-		bytes_sent_count += cnt;
-}
+#define printed(c) do {int cnt=(c); if (cnt > 0)			\
+					   bytes_sent_count += cnt;	\
+	} while(0)
 
-static void printchar(int ch)
-{
-	putchar(ch);
-	printed(1);
-}
+#define printchar(ch) do { putchar((ch)); printed(1); } while(0);
 
 /* POP3 STAT */
 
@@ -1066,6 +1059,14 @@ char	*p;
 		exit(1);
 	}
 
+	if (    auth_getoptionenvint("disableinsecurepop3")
+	    && ((p=getenv("POP3_TLS")) == NULL || !atoi(p)))
+	{
+		printed(printf("-ERR POP3 access disabled via insecure connection.\r\n"));
+		fflush(stdout);
+		exit(1);
+	}
+
 	fprintf(stderr, "INFO: LOGIN, user=%s, ip=[%s], port=[%s]\n",
 			authaddr,
 					remoteip,
