@@ -39,7 +39,7 @@
 
 static const char *recips=0;
 static const char *dbfile=0;
-static char *charset=RFC2045CHARSET;
+static const char *charset;
 static unsigned interval=1;
 static char *sender;
 
@@ -71,14 +71,25 @@ static void usage()
 		charset);
 
 	fprintf(stderr,
+		"    -e                 - Prefer replies to Errors-To: or Return-Path: instead\n"
+		"                         of From:\n"
+		"    -T type            - \"type\": reply, replyall, replydsn, forward, forwardatt\n"
+		"    -N                 - Omit contents of the original message from replies\n"
+		"    -F \"separator\"     - Set the forwarding separator\n"
+		"    -S \"salutation\"    - Set salutation for replies\n"
 		"    -d $pathname       - database to prevent duplicate autoresponses\n"
-		"    -D x               - at least 'x' days before dupes (default: 1)\n"
+		"    -D x               - at least 'x' days before dupes (default: 1)\n");
+
+	fprintf(stderr,
 		"    -s subject         - Subject: on autoresponses\n"
 		"    -A \"Header: stuff\" - Additional header on the autoresponse\n"
-		"    -M recipient       - format autoresponse as a DSN from 'recipient'\n"
+		"    -M recipient       - format \"replydsn\" as a DSN from 'recipient'\n"
+		"    -fuser@domain      - Set responding address for replydsn\n"
+		"    -f                 - Set responding address from $SENDER\n"
 		"    -n                 - only show the resulting message, do not send it\n"
 		"    $MAILER arg arg... - run $MAILER (sendmail) to mail the autoresponse\n"
 		);
+
 	exit(EX_TEMPFAIL);
 }
 
@@ -462,11 +473,16 @@ static void copy_headers(void *ptr)
 	struct mimeautoreply_s *p=(struct mimeautoreply_s *)ptr;
 	char buf[BUFSIZ];
 
+	static const char ct[]="Content-Transfer-Encoding:";
+
 	while (fgets(buf, sizeof(buf), p->contentf) != NULL)
 	{
 		if (buf[0] == '\n')
 			break;
 
+		if (strncasecmp(buf, ct, sizeof(ct)-1) == 0)
+			continue;
+
 		mimeautoreply_write_func(buf, strlen(buf), ptr);
 
 		while (strchr(buf, '\n') == NULL)
@@ -502,18 +518,13 @@ int main(int argc, char **argv)
 	int nosend=0;
 	const char *replymode="reply";
 	int replytoenvelope=0;
+	int donotquote=0;
 	const char *forwardsep="--- Forwarded message ---";
 	const char *replysalut="%F writes:";
-	const struct unicode_info *uinfo;
+	struct rfc2045src *src;
 
 	setlocale(LC_ALL, "");
-	charset=strdup(nl_langinfo(CODESET));
-
-	if (!charset)
-	{
-		perror("malloc");
-		exit(1);
-	}
+	charset=unicode_default_chset();
 
 	sender=NULL;
 	for (argn=1; argn < argc; argn++)
@@ -543,15 +554,19 @@ int main(int argc, char **argv)
 
 			if (optarg && *optarg)
 			{
-				charset=strdup(optarg);
+				char *p=libmail_u_convert_tobuf("",
+								optarg,
+								libmail_u_ucs4_native,
+								NULL);
 
-				if (charset == NULL ||
-				    unicode_find(charset) == NULL)
+				if (!p)
 				{
 					fprintf(stderr, "Unknown charset: %s\n",
 						charset);
 					exit(1);
 				}
+				free(p);
+				charset=optarg;
 			}
 			continue;
 		case 't':
@@ -594,6 +609,9 @@ int main(int argc, char **argv)
 			if (optarg && *optarg)
 				replymode=optarg;
 			continue;
+		case 'N':
+			donotquote=1;
+			continue;
 		case 'F':
 			if (!optarg && argn+1 < argc)
 				optarg=argv[++argn];
@@ -701,11 +719,11 @@ int main(int argc, char **argv)
 	check_db();
 #endif
 
-	uinfo=unicode_find(charset);
+	src=rfc2045src_init_fd(fileno(tmpfp));
 
 	memset(&replyinfo, 0, sizeof(replyinfo));
 
-	replyinfo.info.fd=fileno(tmpfp);
+	replyinfo.info.src=src;
 	replyinfo.info.rfc2045partp=rfcp;
 	replyinfo.info.voidarg=&replyinfo;
 
@@ -717,11 +735,12 @@ int main(int argc, char **argv)
 
 	replyinfo.info.replymode=replymode;
 	replyinfo.info.replytoenvelope=replytoenvelope;
+	replyinfo.info.donotquote=donotquote;
 
 	replyinfo.info.replysalut=replysalut;
 	replyinfo.info.forwarddescr="Forwarded message";
 	replyinfo.info.mailinglists="";
-	replyinfo.info.charset=(uinfo ? uinfo:&unicode_ISO8859_1)->chset;
+	replyinfo.info.charset=charset;
 	replyinfo.info.subject=subj;
 	replyinfo.info.forwardsep=forwardsep;
 
@@ -739,6 +758,61 @@ int main(int argc, char **argv)
 			exit(1);
 		}
 
+		{
+			struct rfc2045 *rfcp=rfc2045_alloc();
+			static const char mv[]="Mime-Version: 1.0\n";
+			char buf[BUFSIZ];
+			int l;
+			const char *content_type;
+			const char *content_transfer_encoding;
+			const char *charset;
+
+			rfc2045_parse(rfcp, mv, sizeof(mv)-1);
+
+			while ((l=fread(buf, 1, sizeof(buf), replyinfo.contentf)
+				) > 0)
+			{
+				rfc2045_parse(rfcp, buf, l);
+			}
+
+			if (l < 0 ||
+			    fseek(replyinfo.contentf, 0L, SEEK_SET) < 0)
+			{
+				perror(mimefile);
+				exit(1);
+			}
+
+			rfc2045_mimeinfo(rfcp, &content_type,
+					 &content_transfer_encoding,
+					 &charset);
+
+			if (strcasecmp(content_type, "text/plain"))
+			{
+				fprintf(stderr,
+					"%s must specify text/plain MIME type\n",
+					mimefile);
+				exit(1);
+			}
+			{
+				char *p=NULL;
+
+				if (charset)
+					p=libmail_u_convert_tobuf("",
+								  charset,
+								  libmail_u_ucs4_native,
+								  NULL);
+
+				if (!p)
+				{
+					fprintf(stderr, "Unknown charset in %s\n",
+						mimefile);
+					exit(1);
+				}
+				free(p);
+				replyinfo.info.charset=strdup(charset);
+			}
+			rfc2045_free(rfcp);
+		}
 		replyinfo.info.content_set_charset=copy_headers;
 		replyinfo.info.content_specify=copy_body;
 	}
@@ -778,7 +852,7 @@ int main(int argc, char **argv)
 		"Precedence: junk\n"
 		"Auto-Submitted: auto-replied\n");
 
-	if (rfc2045_makereply_unicode(&replyinfo.info) < 0 ||
+	if (rfc2045_makereply(&replyinfo.info) < 0 ||
 	    fflush(replyinfo.outf) < 0 || ferror(replyinfo.outf) ||
 	    (!nosend &&
 	     (
@@ -792,8 +866,8 @@ int main(int argc, char **argv)
 	fclose(replyinfo.outf);
 	fcntl(0, F_SETFD, 0);
 
-	free(charset);
 	rfc2045_free(rfcp);
+	rfc2045src_deinit(src);
 
 	if (!nosend)
 		opensendmail(argn, argc, argv);
