@@ -1,11 +1,10 @@
 /*
-** Copyright 1998 - 2009 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2011 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 
 /*
-** $Id: attachments.c,v 1.49 2009/11/08 18:14:47 mrsam Exp $
 */
 #include	"config.h"
 #include	"sqwebmail.h"
@@ -103,7 +102,6 @@ const char	*quotaerr=getarg("QUOTAERR");
 const char	*limiterr=getarg("LIMITERR");
 off_t	dummy;
 int	fd2;
-const struct unicode_info *uiptr=unicode_find(sqwebmail_content_charset);
 
 	CHECKFILENAME(draft);
 	filename=maildir_find(INBOX "." DRAFTS, draft);
@@ -191,8 +189,10 @@ const struct unicode_info *uiptr=unicode_find(sqwebmail_content_charset);
 		printf("<tr><td align=\"left\"><input type=\"checkbox\" name=\"del%d\" id=\"del%d\" />&nbsp;",
 			cnt, cnt);
 
-		if (rfc2231_udecodeType(q, "name", uiptr, &content_name) < 0 ||
-		    rfc2231_udecodeDisposition(q, "filename", uiptr,
+		if (rfc2231_udecodeType(q, "name", sqwebmail_content_charset,
+					&content_name) < 0 ||
+		    rfc2231_udecodeDisposition(q, "filename",
+					       sqwebmail_content_charset,
 					       &content_name) < 0)
 			content_name=NULL;
 
@@ -204,7 +204,7 @@ const struct unicode_info *uiptr=unicode_find(sqwebmail_content_charset);
 			content_name =
 				rfc822_display_hdrvalue_tobuf("subject",
 							      cn,
-							      uiptr->chset,
+							      sqwebmail_content_charset,
 							      NULL,
 							      NULL);
 		}
@@ -912,19 +912,10 @@ int attach_upload(const char *draft,
 		const char *pp;
 
 		argvec[2]=(char *)calc_mime_type(cgi_attachfilename);
-		n=3;
-
-		if (strncasecmp(argvec[2], "text/", 5) == 0 ||
-		    strcasecmp(argvec[2], "auto") == 0)
-		{
-			argvec[3]="-C";
-			argvec[4]=(char *)sqwebmail_content_charset;
-			n=5;
-		}
-
-		argvec[n++]="-N";
-		argvec[n++]=cgi_attachfilename ?
+		argvec[3]="-N";
+		argvec[4]=cgi_attachfilename ?
 			(char *)cgi_attachfilename:"filename.dat";
+		n=5;
 
 		pp=*cgi("attach_inline") ?
 			"Content-Disposition: inline":
@@ -943,6 +934,9 @@ int attach_upload(const char *draft,
 		}
 	}
 
+	argvec[n++]="-C";
+	argvec[n++]=(char *)sqwebmail_content_charset;
+
 	signal(SIGCHLD, SIG_DFL);
 
 	argvec[n++]="-";
