@@ -1,11 +1,10 @@
 /*
-** Copyright 1998 - 2009 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2011 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 
 /*
-** $Id: folder.c,v 1.188 2009/11/18 03:38:50 mrsam Exp $
 */
 #include	"config.h"
 #include	<stdio.h>
@@ -23,8 +22,6 @@
 #include	"rfc822/rfc822.h"
 #include	"rfc822/rfc2047.h"
 #include	"rfc2045/rfc2045.h"
-#include	"rfc2045/rfc2646.h"
-#include	"rfc2646html.h"
 #include	"md5/md5.h"
 #include	"gpglib/gpglib.h"
 #include	"maildir.h"
@@ -167,6 +164,10 @@ const char *f;
 const char *inbox_lab, *drafts_lab, *trash_lab, *sent_lab;
 
 	lab=getarg("FOLDERTITLE");
+
+	if (*cgi("search"))
+		lab=getarg("SEARCHRESULTSTITLE");
+
 	inbox_lab=getarg("INBOX");
 	drafts_lab=getarg("DRAFTS");
 	trash_lab=getarg("TRASH");
@@ -247,7 +248,7 @@ static int groupmove(const char *folder, const char *file, size_t pos)
 	return (maildir_msgmovefile(folder, file, cgi("moveto"), pos));
 }
 
-void folder_delmsgs(const char *dir, size_t pos)
+static const char *do_folder_delmsgs(const char *dir, size_t pos)
 {
 	int	rc=0;
 	char	buf[2];
@@ -256,12 +257,21 @@ void folder_delmsgs(const char *dir, size_t pos)
 	strcpy(buf, ACL_DELETEMSGS);
 	acl_computeRightsOnFolder(dir, buf);
 
-	if (buf[0] == 0)
+#if 0
 	{
-		http_redirect_argu("&error=nodel&form=folder&pos=%s",
-				   (unsigned long)pos);
-		return;
+		FILE *fp=fopen("/tmp/pid", "w");
+
+		if (fp)
+		{
+			fprintf(fp, "%d", (int)getpid());
+			fclose(fp);
+			sleep(10);
+		}
 	}
+#endif
+
+	if (buf[0] == 0)
+		return "nodel";
 
 	if (*cgi("cmddel"))
 	{
@@ -274,18 +284,12 @@ void folder_delmsgs(const char *dir, size_t pos)
 	    struct maildir_info minfo;
 
 	    if (maildir_info_imap_find(&minfo, dir, login_returnaddr())<0) 
-	    {
-		http_redirect_argu("&error=othererror&form=folder&pos=%s",
-			(unsigned long)pos);
-		return ;
-	    }
+		    return "othererror";
 
 	    if ((deldir=maildir_name2dir(minfo.homedir, minfo.maildir)) == NULL)
 	    {
 		maildir_info_destroy(&minfo);
-		http_redirect_argu("&error=othererror&form=folder&pos=%s",
-			(unsigned long)pos);
-		return;
+		return "othererror";
 	    }
 	    
 	    cur = malloc(strlen(deldir)+5);
@@ -308,11 +312,7 @@ void folder_delmsgs(const char *dir, size_t pos)
 		strcpy(buf, ACL_INSERT);
 		acl_computeRightsOnFolder(p, buf);
 		if (buf[0] == 0)
-		{
-			http_redirect_argu("&error=noinsert&form=folder&pos=%s",
-					   (unsigned long)pos);
-			return;
-		}
+			return "noinsert";
 
 		rc=group_movedel( dir, &groupmove );
 		maildir_savefoldermsgs(dir);
@@ -320,11 +320,21 @@ void folder_delmsgs(const char *dir, size_t pos)
 
 	maildir_cleanup();
 
-	http_redirect_argu(
-		(rc ? "&error=quota&form=folder&pos=%s":"&form=folder&pos=%s"),
-			(unsigned long)pos);
+	return rc ? "quota":"";
 }
 
+void folder_delmsgs(const char *dir, size_t pos)
+{
+	const char *status=do_folder_delmsgs(dir, pos);
+	
+	if (*cgi("search"))
+		http_redirect_argsss("&error=%s&form=folder&pos=%s&search=1&"
+				     SEARCHRESFILENAME "=%s", status,
+				     cgi("pos"), cgi(SEARCHRESFILENAME));
+	else
+		http_redirect_argss("&error=%s&form=folder&pos=%s", status,
+				    cgi("pos"));
+}
 
 static void savepath(const char *path, const char *maildir)
 {
@@ -381,25 +391,43 @@ static void savepath(const char *path, const char *maildir)
 	maildir_tmpcreate_free(&createInfo);
 }
 
-static void folder_msg_link(const char *, size_t, char);
-static void folder_msg_unlink(const char *, size_t, char);
+void folder_search(const char *dir, size_t pos)
+{
+	maildir_reload(dir);
+	maildir_search(dir,
+		       pos,
+		       cgi("searchtxt"),
+		       sqwebmail_content_charset,
+		       pref_flagpagesize);
 
-static char *truncate_at(const char *, const struct unicode_info *, size_t);
+	http_redirect_argss("&search=1&form=folder&pos=%s&"
+			    SEARCHRESFILENAME "=%s", cgi("pos"),
+			    cgi(SEARCHRESFILENAME));
+}
 
+static void folder_msg_link(const char *, int, size_t, char);
+static void folder_msg_unlink(const char *, int, size_t, char);
+
+static char *truncate_at(const char *, const char *, size_t);
+
+static void show_msg(const char *dir,
+		     MSGINFO *msg,
+		     MATCHEDSTR *matches,
+		     int row,
+		     const char *charset);
 
 void folder_contents(const char *dir, size_t pos)
 {
 MSGINFO	**contents;
+MATCHEDSTR **matches;
 int	i, found;
 int	morebefore, moreafter;
 const char	*nomsg, *selectalllab;
 const char	*unselectalllab, *golab;
 const char	*qerrmsg;
 long highend;
-const struct unicode_info *uiptr=unicode_find(sqwebmail_content_charset);
-
-	if (!uiptr)
-		uiptr=&unicode_ISO8859_1;
+unsigned long last_message_searched=0;
+unsigned long *last_message_searched_ptr=NULL;
 
 	qerrmsg=getarg("PERMERR");
 
@@ -422,8 +450,41 @@ const struct unicode_info *uiptr=unicode_find(sqwebmail_content_charset);
 	}
 
 	maildir_reload(dir);
-	contents=maildir_read(dir, pref_flagpagesize, &pos,
-		&morebefore, &moreafter);
+
+	if (*cgi("search"))
+	{
+#if 0
+	{
+		FILE *fp=fopen("/tmp/pid", "w");
+
+		if (fp)
+		{
+			fprintf(fp, "%d", (int)getpid());
+			fclose(fp);
+			sleep(10);
+		}
+	}
+#endif
+
+		morebefore=0;
+		moreafter=0;
+		maildir_loadsearch(pref_flagpagesize, &contents, &matches,
+				   &last_message_searched);
+
+		for (i=0; i<pref_flagpagesize; i++)
+		{
+			if (contents[i] == 0)	continue;
+
+			last_message_searched_ptr= &last_message_searched;
+			break;
+		}
+	}
+	else
+	{
+		contents=maildir_read(dir, pref_flagpagesize, &pos,
+				      &morebefore, &moreafter);
+		matches=NULL;
+	}
 
 	time(&current_time);
 	nomsg=getarg("NOMESSAGES");
@@ -439,7 +500,8 @@ const struct unicode_info *uiptr=unicode_find(sqwebmail_content_charset);
 
 	if (!qerrmsg)	qerrmsg="";
 
-	folder_navigate(dir, pos, highend, morebefore, moreafter);
+	folder_navigate(dir, pos, highend, morebefore, moreafter,
+			last_message_searched_ptr);
 
 	printf("<table width=\"100%%\" border=\"0\" cellspacing=\"0\" cellpadding=\"4\"><tr class=\"folder-index-header\"><th align=\"center\">%s</th><th>&nbsp;</th><th>%s</th><th>%s</th><th>%s</th><th>%s</th></tr>\n",
 		getarg("NUM"),
@@ -453,94 +515,11 @@ const struct unicode_info *uiptr=unicode_find(sqwebmail_content_charset);
 	found=0;
 	for (i=0; i<pref_flagpagesize; i++)
 	{
-	const char *date, *from, *subj, *size;
-	char	*froms, *subjs;
-	const char *p, *q;
-	size_t l;
-	char type[8];
-	char *folder_index_entry_start, *folder_index_entry_end;
-
 		if (contents[i] == 0)	continue;
 		found=1;
 
-		date=MSGINFO_DATE(contents[i]);
-		from=MSGINFO_FROM(contents[i]);
-		subj=MSGINFO_SUBJECT(contents[i]);
-		size=MSGINFO_SIZE(contents[i]);
-
-		type[0]=maildirfile_type(MSGINFO_FILENAME(contents[i]));
-		type[1]='\0';
-		if (type[0] == '\0')	strcpy(type, "&nbsp;");
-
-		folder_index_entry_start="<span class=\"read-message\">";
-		folder_index_entry_end="</span>";
-
-		if (type[0] == MSGTYPE_NEW)
-		{
-			folder_index_entry_start="<strong class=\"unread-message\">";
-			folder_index_entry_end="</strong>";
-		}
-
-		p=MSGINFO_FILENAME(contents[i]);
-
-		if ((q=strrchr(p, '/')) != 0)
-			p=q+1;
-
-		printf("<tr class=\"folder-index-bg-%d\" id=\"row%d\"><td align=\"right\" class=\"message-number\">%s%ld.%s</td><td class=\"message-status\"><input type=\"checkbox\" name=\"MOVE-%ld\" id=\"MOVE-%ld",
-		       (i & 1)+1,
-		       i,
-		       folder_index_entry_start,
-		       (long)(i+pos+1),
-		       folder_index_entry_end,
-		       (long) (pos+i),
-		       (long) (pos+i));
-		printf("\" onchange=\"setsel('MOVE-%ld', 'row%d', 'folder-index-bg-%d');\"%s /><input type=\"hidden\" name=\"MOVEFILE-%ld\" value=\"",
-		       (long)(pos+i), i, (i & 1)+1,
-			(type[0] == MSGTYPE_DELETED ? " disabled=\"disabled\"":""),
-			(long)(pos+i));
-		output_attrencoded(p);
-		printf("\" />&nbsp;%s%s%s</td><td class=\"message-date\">%s",
-		       folder_index_entry_start,
-		       type,
-		       folder_index_entry_end,
-
-			folder_index_entry_start
-			);
-		if (!*date)	date=" ";
-		folder_msg_link(dir, pos+i, type[0]);
-		print_safe(date);
-		folder_msg_unlink(dir, pos+i, type[0]);
-		printf("%s</td><td class=\"message-from\">%s", folder_index_entry_end,
-				folder_index_entry_start);
-		if (!*from)	from=" ";
-		folder_msg_link(dir, pos+i, type[0]);
-
-
-		froms=truncate_at(from, uiptr, 30);
-
-		if (froms == 0)	enomem();
-
-		print_safe(froms);
-		free(froms);
-		folder_msg_unlink(dir, pos+i, type[0]);
-		printf("%s<br /></td><td class=\"message-subject\">%s", folder_index_entry_end,
-				folder_index_entry_start);
-
-		folder_msg_link(dir, pos+i, type[0]);
-
-
-		subjs=truncate_at(subj, uiptr, 40);
-
-		if (subjs == 0)	enomem();
-
-		print_safe(subjs);
-		l=strlen(subjs);
-		while (l++ < 8)
-			printf("&nbsp;");
-		free(subjs);
-
-		folder_msg_unlink(dir, pos+i, type[0]);
-		printf("%s</td><td align=\"right\" class=\"message-size\">%s%s&nbsp;%s<br /></td></tr>\n", folder_index_entry_end, folder_index_entry_start, size, folder_index_entry_end);
+		show_msg(dir, contents[i], matches ? matches[i]:NULL, i,
+			 sqwebmail_content_charset);
 	}
 
 	if (found)
@@ -574,7 +553,8 @@ const struct unicode_info *uiptr=unicode_find(sqwebmail_content_charset);
 		puts("</td></tr>");
 
 		printf("</table>\n");
-		folder_navigate(dir, pos, highend, morebefore, moreafter);
+		folder_navigate(dir, pos, highend, morebefore, moreafter,
+				last_message_searched_ptr);
 	}
 	if (!found && nomsg)
 	{
@@ -585,13 +565,173 @@ const struct unicode_info *uiptr=unicode_find(sqwebmail_content_charset);
 	}
 
 	maildir_free(contents, pref_flagpagesize);
+	if (matches)
+		matches_free(matches, pref_flagpagesize);
 }
 
-void folder_navigate(const char *dir, size_t pos, long highend, int morebefore, int moreafter)
+
+static int folder_searchlink()
 {
-const char	*firstlab, *lastlab;
-const char	*beforelab, *afterlab;
-const char	*showncountlab, *jumplab, *golab;
+	if (*cgi("search") == 0)
+		return 0;
+
+	printf("&amp;search=1&amp;" SEARCHRESFILENAME "=");
+	output_urlencoded(cgi(SEARCHRESFILENAME));
+	return 1;
+}
+
+static void show_msg_match_str(const char *prefix,
+			       const char *utf8match,
+			       const char *suffix,
+			       const char *classname);
+
+static void show_msg(const char *dir,
+		     MSGINFO *msg,
+		     MATCHEDSTR *matches,
+		     int row,
+		     const char *charset)
+{
+	const char *date, *from, *subj, *size;
+	char	*froms, *subjs;
+	const char *p, *q;
+	size_t l;
+	char type[8];
+	char *folder_index_entry_start, *folder_index_entry_end;
+
+	size_t msgnum=msg->msgnum;
+
+	date=MSGINFO_DATE(msg);
+	from=MSGINFO_FROM(msg);
+	subj=MSGINFO_SUBJECT(msg);
+	size=MSGINFO_SIZE(msg);
+
+	type[0]=maildirfile_type(MSGINFO_FILENAME(msg));
+	type[1]='\0';
+	if (type[0] == '\0')	strcpy(type, "&nbsp;");
+
+	folder_index_entry_start="<span class=\"read-message\">";
+	folder_index_entry_end="</span>";
+
+	if (type[0] == MSGTYPE_NEW)
+	{
+		folder_index_entry_start="<strong class=\"unread-message\">";
+		folder_index_entry_end="</strong>";
+	}
+
+	p=MSGINFO_FILENAME(msg);
+
+	if ((q=strrchr(p, '/')) != 0)
+		p=q+1;
+
+	printf("<tr class=\"folder-index-bg-%d\" id=\"row%d\"><td align=\"right\" class=\"message-number\">%s%ld.%s</td><td class=\"message-status\"><input type=\"checkbox\" name=\"MOVE-%ld\" id=\"MOVE-%ld",
+	       (row & 1)+1,
+	       row,
+	       folder_index_entry_start,
+	       (long)(msgnum+1),
+	       folder_index_entry_end,
+	       (long) (msgnum),
+	       (long) (msgnum));
+	printf("\" onchange=\"setsel('MOVE-%ld', 'row%d', 'folder-index-bg-%d');\"%s /><input type=\"hidden\" name=\"MOVEFILE-%ld\" value=\"",
+	       (long)(msgnum), row, (row & 1)+1,
+	       (type[0] == MSGTYPE_DELETED ? " disabled=\"disabled\"":""),
+	       (long)(msgnum));
+	output_attrencoded(p);
+	printf("\" />&nbsp;%s%s%s</td><td class=\"message-date\">%s",
+	       folder_index_entry_start,
+	       type,
+	       folder_index_entry_end,
+
+	       folder_index_entry_start
+	       );
+	if (!*date)	date=" ";
+	folder_msg_link(dir, row, msgnum, type[0]);
+	print_safe(date);
+	folder_msg_unlink(dir, row, msgnum, type[0]);
+	printf("%s</td><td class=\"message-from\">%s", folder_index_entry_end,
+	       folder_index_entry_start);
+	if (!*from)	from=" ";
+	folder_msg_link(dir, row, msgnum, type[0]);
+
+
+	froms=truncate_at(from, charset, 30);
+
+	if (froms == 0)	enomem();
+
+	print_safe(froms);
+	free(froms);
+	folder_msg_unlink(dir, row, msgnum, type[0]);
+	printf("%s<br /></td><td class=\"message-subject\">%s", folder_index_entry_end,
+	       folder_index_entry_start);
+
+	folder_msg_link(dir, row, msgnum, type[0]);
+
+#if 0
+	{
+		static int foo=0; if (foo++ == 0)
+	{
+		FILE *fp=fopen("/tmp/pid", "w");
+
+		if (fp)
+		{
+			fprintf(fp, "%d", (int)getpid());
+			fclose(fp);
+			sleep(10);
+		}
+	}
+	}
+#endif
+
+	subjs=truncate_at(subj, charset, 40);
+
+	if (subjs == 0)	enomem();
+
+	print_safe(subjs);
+	l=strlen(subjs);
+	while (l++ < 8)
+		printf("&nbsp;");
+	free(subjs);
+
+	folder_msg_unlink(dir, row, msgnum, type[0]);
+	printf("%s</td><td align=\"right\" class=\"message-size\">%s%s&nbsp;%s<br /></td></tr>\n", folder_index_entry_end, folder_index_entry_start, size, folder_index_entry_end);
+
+	while (matches && matches->match)
+	{
+		printf("<tr class=\"folder-index-bg-%d\"><td align=\"right\" class=\"message-number\" colspan=\"3\">&nbsp;</td><td class=\"message-searchmatch\" colspan=\"3\">", (row & 1)+1);
+		show_msg_match_str("...",
+				   matches->prefix, "", "searchmatch-affix");
+		show_msg_match_str("", matches->match, "", "searchmatch");
+		show_msg_match_str("", matches->suffix,
+				   "...", "searchmatch-affix");
+		printf("</td></tr>\n");
+		++matches;
+	}
+}
+
+static void show_msg_match_str(const char *prefix,
+			       const char *utf8match,
+			       const char *suffix,
+			       const char *classname)
+{
+	char *p;
+
+	p=libmail_u_convert_fromutf8(utf8match, sqwebmail_content_charset,
+				     NULL);
+
+	if (p)
+	{
+		printf("<span class=\"%s\">%s", classname, prefix);
+		output_attrencoded(p);
+		free(p);
+		printf("%s</span>", suffix);
+	}
+}
+
+static void do_folder_navigate(const char *dir, size_t pos, long highend,
+			       int morebefore, int moreafter)
+{
+	const char	*firstlab, *lastlab;
+	const char	*beforelab, *afterlab;
+	const char	*showncountlab, *jumplab, *golab;
 
 	time(&current_time);
 	showncountlab=getarg("SHOWNCOUNT");
@@ -602,9 +742,8 @@ const char	*showncountlab, *jumplab, *golab;
 	afterlab=getarg("NEXTPAGE");
 	lastlab=getarg("LASTPAGE");
 
-	printf("<table width=\"100%%\" class=\"folder-nextprev-background\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\">");
-	printf("<tr><td align=\"left\">");
 	printf("<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" class=\"folder-nextprev-buttons\"><tr><td>");
+
 	if (morebefore)
 	{
 	size_t	beforepos;
@@ -688,11 +827,46 @@ const char	*showncountlab, *jumplab, *golab;
 		printf("</a>");
 
 	printf("</td></tr></table>\n");
-
-	printf("</td></tr></table>");
 }
 
-static void folder_msg_link(const char *dir, size_t pos, char t)
+void folder_navigate(const char *dir, size_t pos, long highend,
+		     int morebefore, int moreafter,
+		     unsigned long *last_message_searched_ptr)
+{
+	printf("<table width=\"100%%\" class=\"folder-nextprev-background\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\"><tr>");
+
+	if (*cgi("search"))
+	{
+		printf("<td class=\"folder-return-link\">");
+		printf("<a href=\"");
+		output_scriptptrget();
+		printf("&amp;form=folder&amp;pos=%ld\">%s</a>",
+		       (long)pos, getarg("RETURNTOFOLDER"));
+		printf("</td>");
+
+		printf("<td class=\"folder-last-message-searched\">");
+		if (last_message_searched_ptr)
+		{
+			printf(getarg("LASTMESSAGESEARCHED"),
+			       *last_message_searched_ptr+1);
+		}
+		else
+		{
+			printf("&nbsp;");
+		}
+		printf("</td>");
+	}
+	else
+	{
+		printf("<td align=\"left\">");
+		do_folder_navigate(dir, pos, highend, morebefore, moreafter);
+		printf("</td>");
+	}
+
+	printf("</tr></table>");
+}
+
+static void folder_msg_link(const char *dir, int row, size_t pos, char t)
 {
 #if 0
 	if (t == MSGTYPE_DELETED)
@@ -708,6 +882,9 @@ static void folder_msg_link(const char *dir, size_t pos, char t)
 	if (strcmp(dir, INBOX "." DRAFTS))
 	{
 		output_scriptptrget();
+		if (folder_searchlink())
+			printf("&amp;searchrow=%d", row);
+
 		printf("&amp;form=readmsg&amp;pos=%ld\">", (long)pos);
 	}
 	else
@@ -725,7 +902,7 @@ static void folder_msg_link(const char *dir, size_t pos, char t)
 	}
 }
 
-static void folder_msg_unlink(const char *dir, size_t pos, char t)
+static void folder_msg_unlink(const char *dir, int row, size_t pos, char t)
 {
 	printf("</a>");
 }
@@ -733,6 +910,10 @@ static void folder_msg_unlink(const char *dir, size_t pos, char t)
 size_t	msg_pos, msg_count;
 static char	*msg_posfile;
 static int	msg_hasprev, msg_hasnext;
+static size_t	msg_searchpos;
+static long	msg_prevpos, msg_prev_searchpos;
+static long	msg_nextpos, msg_next_searchpos;
+
 static const char	*msg_nextlab=0, *msg_prevlab=0, *msg_deletelab=0,
 		*msg_purgelab=0, *msg_folderlab=0;
 static char	msg_type;
@@ -765,6 +946,11 @@ void folder_initnextprev(const char *dir, size_t pos)
 	static char *filename=0;
 	int fd;
 
+	MSGINFO *recp;
+	MSGINFO **search_contents=NULL;
+	MATCHEDSTR **search_matches=NULL;
+	unsigned long last_message_searched=0;
+
 	cgi_put(MIMEGPGFILENAME, "");
 
 	if (filename)
@@ -863,21 +1049,68 @@ void folder_initnextprev(const char *dir, size_t pos)
 	msg_golab=getarg("GOLAB");
 	msg_add=getarg("QUICKADD");
 
-	info=maildir_read(dir, 1, &pos, &msg_hasprev, &msg_hasnext);
-	msg_pos=pos;
+	msg_searchpos=atol(cgi("searchrow"));
 
-	p=strrchr(MSGINFO_FILENAME(info[0]), '/');
+	info=maildir_read(dir, 1, &pos, &msg_hasprev, &msg_hasnext);
+
+	recp=info[0];
+	msg_pos=pos;
+	msg_prevpos=msg_pos-1;
+	msg_nextpos=msg_pos+1;
+	msg_prev_searchpos=msg_prevpos;
+	msg_next_searchpos=msg_nextpos;
+
+	if (*cgi("search"))
+		maildir_loadsearch(pref_flagpagesize,
+				   &search_contents,
+				   &search_matches,
+				   &last_message_searched);
+
+
+	if (search_contents)
+	{
+		if (msg_searchpos < pref_flagpagesize &&
+		    search_contents[msg_searchpos])
+		{
+			recp=search_contents[msg_searchpos];
+
+			msg_pos=recp->msgnum;
+			if ((msg_hasprev=msg_searchpos > 0 &&
+			     search_contents[msg_searchpos-1]) != 0)
+			{
+				msg_prevpos=search_contents
+					[msg_searchpos-1]->msgnum;
+				msg_prev_searchpos=msg_searchpos-1;
+			}
+
+			if ((msg_hasnext=msg_searchpos + 1 <
+			     pref_flagpagesize &&
+			     search_contents[msg_searchpos+1]) != 0)
+			{
+				msg_nextpos=search_contents
+					[msg_searchpos+1]->msgnum;
+				msg_next_searchpos=msg_searchpos+1;
+			}
+
+		}
+	}
+
+	p=strrchr(MSGINFO_FILENAME(recp), '/');
 	if (p)	p++;
-	else	p=MSGINFO_FILENAME(info[0]);
+	else	p=MSGINFO_FILENAME(recp);
 	msg_posfile=strdup(p);
 	if (!msg_posfile)	enomem();
 
-	if ((msg_type=maildirfile_type(MSGINFO_FILENAME(info[0])))
+	if ((msg_type=maildirfile_type(MSGINFO_FILENAME(recp)))
 		== MSGTYPE_NEW) msg_numlab=msg_numnewlab;
 
 	msg_msglab=msg_numlab;
 	msg_count=maildir_countof(dir);
 	maildir_free(info, 1);
+	if (search_contents)
+		maildir_free(search_contents, pref_flagpagesize);
+	if (search_matches)
+		matches_free(search_matches, pref_flagpagesize);
 }
 
 char *get_msgfilename(const char *folder, size_t *pos)
@@ -916,7 +1149,7 @@ void folder_nextprev()
 {
 	printf("<table width=\"100%%\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\" class=\"message-menu-background\"><tr valign=\"middle\">");
 
-    printf("<td align=\"left\"><table border=\"0\" cellspacing=\"4\" cellpadding=\"4\"><tr valign=\"top\">");
+	printf("<td align=\"left\"><table border=\"0\" cellspacing=\"4\" cellpadding=\"4\"><tr valign=\"top\">");
 
 	/* PREV */
 
@@ -926,8 +1159,12 @@ void folder_nextprev()
 	{
 		printf("<a href=\"");
 		output_scriptptrget();
+
+		if (folder_searchlink())
+			printf("&searchrow=%ld", msg_prev_searchpos);
+
 		printf("&amp;form=readmsg&amp;pos=%ld\">",
-			(long)(msg_pos-1));
+		       msg_prevpos);
 	}
 
 	printf("%s", msg_prevlab ? msg_prevlab:"");
@@ -946,8 +1183,12 @@ void folder_nextprev()
 	{
 		printf("<a href=\"");
 		output_scriptptrget();
+
+		if (folder_searchlink())
+			printf("&searchrow=%ld", msg_next_searchpos);
+
 		printf("&amp;form=readmsg&amp;pos=%ld\">",
-			(long)(msg_pos+1));
+		       msg_nextpos);
 	}
 
 	printf("%s", msg_nextlab ? msg_nextlab:"");
@@ -966,6 +1207,7 @@ void folder_nextprev()
 		printf("<a href=\"");
 		output_scriptptrget();
 		tokennewget();
+		folder_searchlink();
 		printf("&amp;posfile=");
 		output_urlencoded(msg_posfile);
 		printf("&amp;form=delmsg&amp;pos=%ld\">",
@@ -983,6 +1225,7 @@ void folder_nextprev()
 
 	printf("<td class=\"message-menu-button\"><a href=\"");
 	output_scriptptrget();
+	folder_searchlink();
 	printf("&amp;pos=%ld&amp;form=folder\">%s</a></td>\n",
 		(long)( (msg_pos/pref_flagpagesize)*pref_flagpagesize ),
 		msg_folderlab);
@@ -1041,6 +1284,7 @@ void folder_nextprev()
 	{
 		printf("<td class=\"message-menu-button\"><a href=\"");
 		output_scriptptrget();
+		folder_searchlink();
 		output_mimegpgfilename();
 		printf("&amp;pos=%ld&amp;form=readmsg&amp;fullheaders=1\">%s</a></td>\n",
 			(long)msg_pos, msg_fullheaderlab);
@@ -1281,6 +1525,7 @@ int	dummy;
 const	char *f=cgi("posfile");
 size_t	newpos;
 int	rc=0;
+char nbuf[MAXLONGSIZE+10];
 
 	CHECKFILENAME(f);
 
@@ -1312,15 +1557,24 @@ int	rc=0;
 
 	if (info[0] && newpos != pos)
 	{
-		maildir_free(info, 1);
-		http_redirect_argu("&form=readmsg&pos=%s",
-						(unsigned long)newpos);
+		sprintf(nbuf, "%lu", (unsigned long)newpos);
 	}
 	else
 	{
-		maildir_free(info, 1);
-		http_redirect_argu("&form=folder&pos=%s",
-						(unsigned long)pos);
+		sprintf(nbuf, "%lu", (unsigned long)pos);
+	}
+
+	maildir_free(info, 1);
+
+	if (*cgi("search"))
+	{
+		http_redirect_argss("&form=readmsg&pos=%s&search=1&"
+				    SEARCHRESFILENAME "=%s", nbuf,
+				    cgi(SEARCHRESFILENAME));
+	}
+	else
+	{
+		http_redirect_argss("&form=readmsg&pos=%s", nbuf, "");
 	}
 }
 
@@ -1333,24 +1587,20 @@ static int is_preview_mode()
 
 static void dokeyimport(FILE *, struct rfc2045 *, int);
 
-void charset_warning(const char *mime_charset)
+static void charset_warning(const char *mime_charset, void *arg)
 {
-	const struct unicode_info *u1, *u2;
+	char charset_buf[32];
+	char *p;
 
-	if (strcasecmp(mime_charset, "us-ascii") == 0)
-		return;
+	charset_buf[0]=0;
+	strncat(charset_buf, mime_charset, sizeof(charset_buf)-1);
 
-	u1=unicode_find(mime_charset);
-	u2=unicode_find(sqwebmail_content_charset);
+	for (p=charset_buf; *p; p++)
+		if (*p < ' ' || *p > 0x7f ||
+		    *p == '<' || *p == '>' || *p == '&')
+			*p=' ';
 
-	if (u2 && (u2->flags & UNICODE_UTF))
-		return;
-
-	if (!u1 || !u2 || strcmp(u1->chset, u2->chset))
-	{
-		printf(getarg("CHSET"), mime_charset,
-		       sqwebmail_content_charset);
-	}
+	printf(getarg("CHSET"), charset_buf, sqwebmail_content_charset);
 }
 
 static void html_warning()
@@ -1476,50 +1726,39 @@ static const char *email_header_date_fmt(const char *def)
 	return def;
 }
 
-extern const char *redirect_hash(const char *timestamp);
-
-static char *decode_cgiurlencode(const char *s)
+static void buf_cat_esc_amp(struct buf *b, const char *url)
 {
-char *q=malloc(strlen(s)+1), *r;
-const char *t;
-
-	if (!q)	enomem();
-
-	for (r=q, t=s; *t; )
+	for (; *url; ++url)
 	{
-		if (strncmp(t, "&amp;", 5) == 0)
-		{
-			*r++ = '&';
-			t += 5;
-			continue;
-		}
-		if (strncmp(t, "&lt;", 4) == 0)
-		{
-			*r++ = '<';
-			t += 4;
-			continue;
-		}
-		if (strncmp(t, "&gt;", 4) == 0)
-		{
-			*r++ = '>';
-			t += 4;
-			continue;
-		}
-		if (strncmp(t, "&quot;", 6) == 0)
-		{
-			*r++ = '"';
-			t += 6;
-			continue;
-		}
-		*r++ = *t++;
-	}
-	*r=0;
+		char	c[2];
 
-	r=cgiurlencode(q);
-	free(q);
-	return (r);
+		if (*url == '&')
+		{
+			buf_cat(b, "&amp;");
+		}
+		else if (*url == '<')
+		{
+			buf_cat(b, "&lt;");
+		}
+		else if (*url == '>')
+		{
+			buf_cat(b, "&gt;");
+		}
+		else if (*url == '"')
+		{
+			buf_cat(b, "&quot;");
+		}
+		else
+		{
+			c[0]=*url;
+			c[1]=0;
+			buf_cat(b, c);
+		}
+	}
 }
 
+extern const char *redirect_hash(const char *timestamp);
+
 static char *get_textlink(const char *s, void *arg)
 {
 char	*t;
@@ -1532,7 +1771,14 @@ struct buf b;
 	int	i;
 
 		buf_cat(&b, "<a href=\"");
-		buf_cat(&b, scriptptrget());
+
+		{
+			char *p=scriptptrget();
+
+			buf_cat(&b, p);
+			free(p);
+		}
+
 		buf_cat(&b, "&amp;form=newmsg&amp;to=");
 
 		for (i=7; s[i]; i++)
@@ -1542,11 +1788,30 @@ struct buf b;
 			c[1]=0;
 			if ((c[0]=s[i]) == '?')
 				c[0]='&';
-			buf_cat(&b, c);
+ 			else if (c[0] == '&')
+			{
+				buf_cat(&b, "&amp;");
+			}
+			else if (c[0] == '<')
+			{
+				buf_cat(&b, "&lt;");
+			}
+			else if (c[0] == '>')
+			{
+				buf_cat(&b, "&gt;");
+			}
+			else if (c[0] == '"')
+			{
+				buf_cat(&b, "&quot;");
+			}
+			else
+			{
+				buf_cat(&b, c);
+			}
 		}
 		buf_cat(&b, "\">"
 			"<span class=\"message-text-plain-mailto-link\">");
-		buf_cat(&b, s);
+		buf_cat_esc_amp(&b, s);
 		buf_cat(&b, "</span></a>");
 	}
 	else if (strncmp(s, "http:", 5) == 0 || strncmp(s, "https:", 6) == 0)
@@ -1561,11 +1826,11 @@ struct buf b;
 
 		hash=cgiurlencode(redirect_hash(buffer));
 
-		t=decode_cgiurlencode(s);
+		t=cgiurlencode(s);
 		buf_cat(&b, "<a href=\"");
 
 		n=getenv("SCRIPT_NAME");
-		if (!*n) n="/";
+		if (!n || !*n) n="/";
 
 		buf_cat(&b, n);
 		buf_cat(&b, "?redirect=");
@@ -1580,7 +1845,7 @@ struct buf b;
 		}
 		buf_cat(&b, "\" target=\"_blank\">"
 			"<span class=\"message-text-plain-http-link\">");
-		buf_cat(&b, s);
+		buf_cat_esc_amp(&b, s);
 		buf_cat(&b, "</span></a>");
 		free(t);
 	}
@@ -1827,16 +2092,16 @@ static char *get_url_to_mime_part(const char *mimeid, void *arg)
 	q=malloc(strlen(p)+strlen(pos) +
 		 strlen(mimegpgfilename)+
 		 strlen(mimeid)+
-		 sizeof("&amp;mimeid=&amp;pos=&amp;form=fetch&amp;mimegpgfilename="));
+		 sizeof("&mimeid=&pos=&form=fetch&mimegpgfilename="));
 	if (!q)	enomem();
 	strcpy(q, p);
-	strcat(q, "&amp;form=fetch&amp;pos=");
+	strcat(q, "&form=fetch&pos=");
 	strcat(q, pos);
-	strcat(q, "&amp;mimeid=");
+	strcat(q, "&mimeid=");
 	strcat(q, mimeid);
 
 	if (*mimegpgfilename)
-		strcat(strcat(q, "&amp;mimegpgfilename="), mimegpgfilename);
+		strcat(strcat(q, "&mimegpgfilename="), mimegpgfilename);
 
 	return (q);
 }
@@ -1911,7 +2176,7 @@ void folder_showmsg(const char *dir, size_t pos)
 		char *washpfix;
 		char *washpfixmailto;
 		char *scriptnameget=scriptptrget();
-		static const char formbuf[]="&amp;form=newmsg&amp;to=";
+		static const char formbuf[]="&form=newmsg&to=";
 
 		info->mimegpgfilename=cgi(MIMEGPGFILENAME);
 		if (*info->mimegpgfilename)
@@ -1922,7 +2187,7 @@ void folder_showmsg(const char *dir, size_t pos)
 		info->noflowedtext=pref_noflowedtext;
 		info->showhtml=pref_showhtml;
 
-		info->character_set_follows=charset_warning;
+		info->charset_warning=charset_warning;
 		info->html_content_follows=html_warning;
 		info->get_url_to_mime_part=get_url_to_mime_part;
 
@@ -1940,9 +2205,9 @@ void folder_showmsg(const char *dir, size_t pos)
 							  script_name),
 						   "?timestamp="),
 					    nowbuffer),
-				     "&amp;md5="),
+				     "&md5="),
 			      (hash ? hash:"")),
-		       "&amp;redirect=");
+		       "&redirect=");
 
 		if (hash)
 			free(hash);
@@ -1950,6 +2215,7 @@ void folder_showmsg(const char *dir, size_t pos)
 		washpfixmailto=malloc(strlen(scriptnameget)+sizeof(formbuf));
 		if (!washpfixmailto)	enomem();
 		strcat(strcpy(washpfixmailto, scriptnameget), formbuf);
+		free(scriptnameget);
 
 		info->wash_http_prefix=washpfix;
 		info->wash_mailto_prefix=washpfixmailto;
@@ -1968,8 +2234,9 @@ void folder_showmsg(const char *dir, size_t pos)
 			application_pgp_keys_action;
 		info->gpg_message_action=gpg_message_action;
 
-		info->is_gpg_enabled=is_gpg_enabled;
-		info->is_preview_mode=is_preview_mode;
+		info->is_gpg_enabled=is_gpg_enabled();
+		info->is_preview_mode=is_preview_mode();
+
 		msg2html(fp, rfc, info);
 		msg2html_free(info);
 
@@ -2350,6 +2617,8 @@ void folder_list()
 		const char	*newfoldername=trim_spaces(cgi("foldername"));
 		const char	*newdirname=trim_spaces(cgi("dirname"));
 		const char	*folderdir=cgi("folderdir");
+		char	*futf7;
+		char	*dutf7;
 
 		/*
 		** New folder names cannot contain .s, and must be considered
@@ -2359,21 +2628,23 @@ void folder_list()
 		if (!*folderdir)
 			folderdir=INBOX;
 
+		futf7=folder_toutf7(newfoldername);
+		dutf7=folder_toutf7(newdirname);;
+
 		if (!*newfoldername ||
-			strchr(newfoldername, '.') ||
-			strchr(newdirname, '.'))
+		    strchr(futf7, '.') ||
+		    strchr(dutf7, '.'))
+		{
+			free(futf7);
+			free(dutf7);
 			folder_err_msg=err_invalid;
+		}
 		else
 		{
 			char	*p;
-			char	*futf7;
-			char	*dutf7;
 			struct maildir_info minfo;
 			char *q;
 
-			futf7=folder_toutf7(newfoldername);
-			dutf7=folder_toutf7(newdirname);;
-
 			p=malloc(strlen(folderdir)+strlen(futf7)
 				 +strlen(dutf7)+3);
 
@@ -3348,6 +3619,16 @@ void folder_showtransfer()
 	folder_sent=getarg("SENT");
 
 	printf("<input type=\"hidden\" name=\"pos\" value=\"%s\" />", cgi("pos"));
+
+	if (*cgi("search"))
+	{
+		printf("<input type=\"hidden\" name=\"search\" value=\"1\" />"
+		       "<input type=\"hidden\" name=\"" SEARCHRESFILENAME
+		       "\" value=\"");
+		output_attrencoded(cgi(SEARCHRESFILENAME));
+		printf("\" />");
+	}
+
 	if ((strcmp(sqwebmail_folder, INBOX "." TRASH) == 0) && (strlen(getarg("PURGEALL"))))
 	    printf("<input type=\"submit\" name=\"cmdpurgeall\" value=\"%s\" onclick=\"javascript: return deleteAll();\" />", 
 		getarg("PURGEALL"));
@@ -3418,18 +3699,30 @@ folder_cleanup()
 */
 
 static char *truncate_at(const char *str,
-			 const struct unicode_info *uiptr,
+			 const char *charset,
 			 size_t ncols)
 {
 	unicode_char *uc;
 	size_t n;
 	size_t cols, tp=0;
 	char *retbuf;
+	libmail_u_convert_handle_t h;
+	int chopped=0;
 
 	if (!str)
 		return NULL;
 
-	uc= (*unicode_UTF8.c2u)(&unicode_UTF8, str, NULL);
+	h=libmail_u_convert_tou_init("utf-8", &uc, &n, 1);
+
+	if (h)
+	{
+		libmail_u_convert(h, str, strlen(str));
+		libmail_u_convert_deinit(h, NULL);
+	}
+	else
+	{
+		uc=NULL;
+	}
 
 	if (!uc)
 		return NULL;
@@ -3438,21 +3731,37 @@ static char *truncate_at(const char *str,
 
 		cols += unicode_wcwidth(uc[n]);
 
-		if (!tp && cols > ncols-3)
-			tp = n;
+		tp = n;
+		if (cols > ncols-3 && n > 0 &&
+		    unicode_grapheme_break(uc[n-1], uc[n]))
+		{
+			chopped=1;
+			break;
+		}
+		++tp;
 	}
 
-	if (cols > ncols)
+	if (chopped)
 	{
 		uc = realloc(uc, sizeof(unicode_char) * (tp+4));
 		if (uc == 0) enomem();
 		uc[tp]='.';
 		uc[tp+1]='.';
 		uc[tp+2]='.';
-		uc[tp+3]=0;
+		tp += 3;
 	}
 
-	retbuf=(*uiptr->u2c)(uiptr, uc, NULL);
+	h=libmail_u_convert_fromu_init(charset, &retbuf, &cols, 1);
+
+	if (h)
+	{
+		libmail_u_convert_uc(h, uc, tp);
+		libmail_u_convert_deinit(h, NULL);
+	}
+	else
+	{
+		retbuf=NULL;
+	}
 
 	free(uc);
 	return retbuf;
